@@ -10,6 +10,7 @@
 #include "Magick++/Include.h"
 #include <string>
 #include <ctype.h> // for isdigit
+#include <string.h> // for strcpy
 
 using namespace std;
 
@@ -198,7 +199,7 @@ Magick::Geometry::operator = ( const std::string &geometry_ )
       if ( pageptr != 0 )
 	{
 	  strcpy(geom,pageptr);
-	  LiberateMemory( reinterpret_cast<void **>(&pageptr) );
+          MagickFreeMemory(pageptr);
 	}
     }
 
