@@ -18,7 +18,7 @@
 %                            BBBB   M   M  P                                  %
 %                                                                             %
 %                                                                             %
-%                   Read/Write GraphicsMagick Image Format.                   %
+%             Read/Write Microsoft Windows Bitmap Image Format.               %
 %                                                                             %
 %                                                                             %
 %                              Software Design                                %
@@ -38,11 +38,13 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/cache.h"
+#include "magick/constitute.h"
+#include "magick/pixel_cache.h"
 #include "magick/color.h"
 #include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
+#include "magick/profile.h"
 #include "magick/transform.h"
 #include "magick/utility.h"
 
@@ -54,7 +56,7 @@
 #define BI_JPEG  4
 #undef BI_PNG
 #define BI_PNG  5
-#if !defined(WIN32) || defined(__MINGW32__)
+#if !defined(MSWINDOWS) || defined(__MINGW32__)
 #define BI_RGB  0
 #define BI_RLE8  1
 #define BI_RLE4  2
@@ -170,6 +172,9 @@ static unsigned int DecodeImage(Image *image,const unsigned long compression,
   register unsigned char
     *q;
 
+  unsigned char
+    *end;
+
   assert(image != (Image *) NULL);
   assert(pixels != (unsigned char *) NULL);
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Decoding RLE pixels");
@@ -177,13 +182,17 @@ static unsigned int DecodeImage(Image *image,const unsigned long compression,
   byte=0;
   x=0;
   q=pixels;
+  end=pixels + (size_t) image->columns*image->rows;
   for (y=0; y < (long) image->rows; )
   {
+    if (q < pixels || q  >= end)
+      break;
     count=ReadBlobByte(image);
     if (count == EOF)
       break;
     if (count != 0)
       {
+        count=Min(count, end - q);
         /*
           Encoded mode.
         */
@@ -240,6 +249,7 @@ static unsigned int DecodeImage(Image *image,const unsigned long compression,
             /*
               Absolute mode.
             */
+    	    count=Min(count, end - q);
             if (compression == BI_RLE8)
               for (i=count; i != 0; --i)
                 *q++=ReadBlobByte(image);
@@ -451,14 +461,10 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
-  IndexPacket
-    index;
-
   int
     logging;
 
   long
-    bit,
     y;
 
   unsigned long
@@ -470,9 +476,6 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ExtendedSignedIntegralType
     start_position;
 
-  register IndexPacket
-    *indexes;
-
   register long
     x;
 
@@ -510,7 +513,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+    ThrowReaderException(FileOpenError,UnableToOpenFile,image);
   /*
     Determine if this is a BMP file.
   */
@@ -546,7 +549,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         magick[0],magick[1]);
     if ((count == 0) || ((LocaleNCompare((char *) magick,"BM",2) != 0) &&
         (LocaleNCompare((char *) magick,"CI",2) != 0)))
-      ThrowReaderException(CorruptImageError,"NotABMPImageFile",image);
+      ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
     bmp_info.file_size=ReadBlobLSBLong(image);
     (void) ReadBlobLSBLong(image);
     bmp_info.offset_bits=ReadBlobLSBLong(image);
@@ -560,8 +563,8 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Windows 2.X or OS/2 BMP image file.
         */
-        bmp_info.width=(short) ReadBlobLSBShort(image);
-        bmp_info.height=(short) ReadBlobLSBShort(image);
+        bmp_info.width=(magick_int16_t) ReadBlobLSBShort(image);
+        bmp_info.height=(magick_int16_t) ReadBlobLSBShort(image);
         bmp_info.planes=ReadBlobLSBShort(image);
         bmp_info.bits_per_pixel=ReadBlobLSBShort(image);
         bmp_info.x_pixels=0;
@@ -583,13 +586,22 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     else
       {
         /*
-          Microsoft Windows BMP image file.
+          Microsoft Windows 3.X or later BMP image file.
         */
         if (bmp_info.size < 40)
-          ThrowReaderException(CorruptImageWarning,"NonOS2HeaderSizeError",
+          ThrowReaderException(CorruptImageWarning,NonOS2HeaderSizeError,
             image);
-        bmp_info.width=(short) ReadBlobLSBLong(image);
-        bmp_info.height=(short) ReadBlobLSBLong(image);
+
+        /*
+          BMP v3 defines width and hight as signed LONG (32 bit) values.  If
+          height is a positive number, then the image is a "bottom-up"
+          bitmap with origin in the lower-left corner.  If height is a
+          negative number, then the image is a "top-down" bitmap with the
+          origin in the upper-left corner.  The meaning of negative values
+          is not defined for width.
+        */
+        bmp_info.width=(magick_int32_t) ReadBlobLSBLong(image);
+        bmp_info.height=(magick_int32_t) ReadBlobLSBLong(image);
         bmp_info.planes=ReadBlobLSBShort(image);
         bmp_info.bits_per_pixel=ReadBlobLSBShort(image);
         bmp_info.compression=ReadBlobLSBLong(image);
@@ -656,14 +668,10 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
               "  Number of colors: %lu",bmp_info.number_colors);
           }
 
-        if ((bmp_info.compression == BI_BITFIELDS) &&
-            ((bmp_info.bits_per_pixel == 16) ||
-             (bmp_info.bits_per_pixel == 32)))
-          {
-            bmp_info.red_mask=ReadBlobLSBLong(image);
-            bmp_info.green_mask=ReadBlobLSBLong(image);
-            bmp_info.blue_mask=ReadBlobLSBLong(image);
-          }
+        bmp_info.red_mask=ReadBlobLSBLong(image);
+        bmp_info.green_mask=ReadBlobLSBLong(image);
+        bmp_info.blue_mask=ReadBlobLSBLong(image);
+
         if (bmp_info.size > 40)
           {
             double
@@ -755,38 +763,42 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
             profile_data=profile_data;
             profile_size=ReadBlobLSBLong(image);
             profile_size=profile_size;
-            ReadBlobLSBLong(image);  /* Reserved byte */
+            (void) ReadBlobLSBLong(image);  /* Reserved byte */
           }
       }
 
     if ((bmp_info.compression != BI_RGB) &&
-        (bmp_info.file_size != GetBlobSize(image)))
-      ThrowReaderException(CorruptImageError,"LengthAndFilesizeDoNotMatch",
+        ((magick_off_t) bmp_info.file_size != GetBlobSize(image)))
+      ThrowReaderException(CorruptImageError,LengthAndFilesizeDoNotMatch,
         image);
     if (bmp_info.width <= 0)
-      ThrowReaderException(CorruptImageWarning,"NegativeOrZeroImageSize",image);
+      ThrowReaderException(CorruptImageWarning,NegativeOrZeroImageSize,image);
     if (bmp_info.height == 0)
-      ThrowReaderException(CorruptImageWarning,"NegativeOrZeroImageSize",image);
+      ThrowReaderException(CorruptImageWarning,NegativeOrZeroImageSize,image);
     if ((bmp_info.height < 0) && (bmp_info.compression !=0))
-      ThrowReaderException(CorruptImageWarning,"CompressionNotValid",image);
+      ThrowReaderException(CorruptImageWarning,CompressionNotValid,image);
     if (bmp_info.planes != 1)
-      ThrowReaderException(CorruptImageWarning,"StaticPlanesValueNotEqualToOne",
+      ThrowReaderException(CorruptImageWarning,StaticPlanesValueNotEqualToOne,
         image);
     if ((bmp_info.bits_per_pixel != 1) && (bmp_info.bits_per_pixel != 4) &&
         (bmp_info.bits_per_pixel != 8) && (bmp_info.bits_per_pixel != 16) &&
         (bmp_info.bits_per_pixel != 24) && (bmp_info.bits_per_pixel != 32))
-      ThrowReaderException(CorruptImageWarning,"UnrecognizedBitsPerPixel",image);
+      ThrowReaderException(CorruptImageWarning,UnrecognizedBitsPerPixel,image);
     if (bmp_info.number_colors > (1UL << bmp_info.bits_per_pixel))
-      ThrowReaderException(CorruptImageWarning,"UnrecognizedNumberOfColors",image);
+    {
+      if (bmp_info.bits_per_pixel<24)
+        ThrowReaderException(CorruptImageWarning,UnrecognizedNumberOfColors,image);
+      bmp_info.number_colors = 0;
+    }
     if (bmp_info.compression > 3)
-      ThrowReaderException(CorruptImageWarning,"UnrecognizedImageCompression",image);
+      ThrowReaderException(CorruptImageWarning,UnrecognizedImageCompression,image);
     if ((bmp_info.compression == 1) && (bmp_info.bits_per_pixel != 8))
-      ThrowReaderException(CorruptImageWarning,"UnrecognizedBitsPerPixel",image);
+      ThrowReaderException(CorruptImageWarning,UnrecognizedBitsPerPixel,image);
     if ((bmp_info.compression == 2) && (bmp_info.bits_per_pixel != 4))
-      ThrowReaderException(CorruptImageWarning,"UnrecognizedBitsPerPixel",image);
+      ThrowReaderException(CorruptImageWarning,UnrecognizedBitsPerPixel,image);
     if ((bmp_info.compression == 3) && (bmp_info.bits_per_pixel < 16))
-      ThrowReaderException(CorruptImageWarning,"UnrecognizedBitsPerPixel",image);
-    switch (bmp_info.compression)
+      ThrowReaderException(CorruptImageWarning,UnrecognizedBitsPerPixel,image);
+    switch ((unsigned int) bmp_info.compression)
     {
       case BI_RGB:
       case BI_RLE8:
@@ -794,11 +806,11 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
       case BI_BITFIELDS:
         break;
       case BI_JPEG:
-        ThrowReaderException(CoderError,"JPEGCompressionNotSupported",image)
+        ThrowReaderException(CoderError,JPEGCompressionNotSupported,image)
       case BI_PNG:
-        ThrowReaderException(CoderError,"PNGCompressionNotSupported",image)
+        ThrowReaderException(CoderError,PNGCompressionNotSupported,image)
       default:
-        ThrowReaderException(CorruptImageError,"UnrecognizedImageCompressionMethod",
+        ThrowReaderException(CorruptImageError,UnrecognizedImageCompression,
           image)
     }
     image->columns=bmp_info.width;
@@ -825,19 +837,20 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         if (logging)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-            "  Reading colormap of %ld colors",image->colors);
+            "  Reading colormap of %u colors",image->colors);
         if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
+          ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
             image);
-        bmp_colormap=(unsigned char *) AcquireMemory(4*image->colors);
+        bmp_colormap=MagickAllocateMemory(unsigned char *,4*image->colors);
         if (bmp_colormap == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
+          ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
             image);
         if ((bmp_info.size == 12) || (bmp_info.size == 64))
           packet_size=3;
         else
           packet_size=4;
-        (void) SeekBlob(image,start_position+14+bmp_info.size,SEEK_SET);
+        if (SeekBlob(image,start_position+14+bmp_info.size,SEEK_SET) == -1)
+	  ThrowReaderException(CorruptImageError,ImproperImageHeader,image)
         (void) ReadBlob(image,packet_size*image->colors,(char *) bmp_colormap);
         p=bmp_colormap;
         for (i=0; i < (long) image->colors; i++)
@@ -848,7 +861,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (packet_size == 4)
             p++;
         }
-        LiberateMemory((void **) &bmp_colormap);
+        MagickFreeMemory(bmp_colormap);
       }
     if (image_info->ping && (image_info->subrange != 0))
       if (image->scene >= (image_info->subimage+image_info->subrange-1))
@@ -856,15 +869,16 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Read image data.
     */
-    (void) SeekBlob(image,start_position+bmp_info.offset_bits,SEEK_SET);
+    if (SeekBlob(image,start_position+bmp_info.offset_bits,SEEK_SET) == -1)
+      ThrowReaderException(CorruptImageError,ImproperImageHeader,image)
     if (bmp_info.compression == BI_RLE4)
       bmp_info.bits_per_pixel<<=1;
     bytes_per_line=4*((image->columns*bmp_info.bits_per_pixel+31)/32);
     length=bytes_per_line*image->rows;
-    pixels=(unsigned char *)
-      AcquireMemory(Max(bytes_per_line,image->columns+1)*image->rows);
+    pixels=MagickAllocateMemory(unsigned char *,
+      Max(bytes_per_line,image->columns+1)*image->rows);
     if (pixels == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
     if ((bmp_info.compression == BI_RGB) ||
         (bmp_info.compression == BI_BITFIELDS))
       {
@@ -880,7 +894,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         status=DecodeImage(image,bmp_info.compression,pixels);
         if (status == False)
-          ThrowReaderException(CorruptImageError,"UnableToRunlengthDecodeImage",
+          ThrowReaderException(CorruptImageError,UnableToRunlengthDecodeImage,
             image);
       }
     /*
@@ -916,8 +930,8 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Get shift and quantum bits info from bitfield masks.
         */
-        memset(&shift,0,sizeof(PixelPacket));
-        memset(&quantum_bits,0,sizeof(PixelPacket));
+        (void) memset(&shift,0,sizeof(PixelPacket));
+        (void) memset(&quantum_bits,0,sizeof(PixelPacket));
         if (bmp_info.red_mask != 0)
           while (((bmp_info.red_mask << shift.red) & 0x80000000UL) == 0)
             shift.red++;
@@ -960,27 +974,9 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          indexes=GetIndexes(image);
-          for (x=0; x < ((long) image->columns-7); x+=8)
-          {
-            for (bit=0; bit < 8; bit++)
-            {
-              index=((*p) & (0x80 >> bit)) ? 0x01 : 0x00;
-              indexes[x+bit]=index;
-              *q++=image->colormap[index];
-            }
-            p++;
-          }
-          if ((image->columns % 8) != 0)
-            {
-              for (bit=0; bit < (long) (image->columns % 8); bit++)
-              {
-                index=((*p) & (0x80 >> bit)) ? 0x01 : 0x00;
-                indexes[x+bit]=index;
-                *q++=image->colormap[index];
-              }
-              p++;
-            }
+          if (ImportImagePixelArea(image,IndexQuantum,bmp_info.bits_per_pixel,p,0,0)
+              == MagickFail)
+            break;
           if (!SyncImagePixels(image))
             break;
           if (image->previous == (Image *) NULL)
@@ -1005,27 +1001,9 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          indexes=GetIndexes(image);
-          for (x=0; x < ((long) image->columns-1); x+=2)
-          {
-            index=(IndexPacket) ((*p >> 4) & 0xf);
-            VerifyColormapIndex(image,index);
-            indexes[x]=index;
-            *q++=image->colormap[index];
-            index=(IndexPacket) (*p & 0xf);
-            VerifyColormapIndex(image,index);
-            indexes[x+1]=index;
-            *q++=image->colormap[index];
-            p++;
-          }
-          if ((image->columns % 2) != 0)
-            {
-              index=(IndexPacket) ((*p >> 4) & 0xf);
-              VerifyColormapIndex(image,index);
-              indexes[x]=index;
-              *q++=image->colormap[index];
-              p++;
-            }
+          if (ImportImagePixelArea(image,IndexQuantum,bmp_info.bits_per_pixel,p,0,0)
+              == MagickFail)
+            break;
           if (!SyncImagePixels(image))
             break;
           if (image->previous == (Image *) NULL)
@@ -1053,16 +1031,9 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          indexes=GetIndexes(image);
-          for (x = (long)image->columns; x != 0; --x)
-          {
-            index=(IndexPacket)*p;
-            VerifyColormapIndex(image,index);
-            *indexes++=index;
-            *q=image->colormap[index];
-            p++;
-            q++;
-          }
+          if (ImportImagePixelArea(image,IndexQuantum,bmp_info.bits_per_pixel,p,0,0)
+              == MagickFail)
+            break;
           if (!SyncImagePixels(image))
             break;
           if (image->previous == (Image *) NULL)
@@ -1086,7 +1057,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         if (bmp_info.compression != BI_RGB &&
             bmp_info.compression != BI_BITFIELDS)
-          ThrowReaderException(CorruptImageError,"UnrecognizedImageCompression",image)
+          ThrowReaderException(CorruptImageError,UnrecognizedImageCompression,image)
         bytes_per_line=2*(image->columns+image->columns%2);
         image->storage_class=DirectClass;
         for (y=(long) image->rows-1; y >= 0; y--)
@@ -1172,7 +1143,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         if ((bmp_info.compression != BI_RGB) &&
             (bmp_info.compression != BI_BITFIELDS))
-          ThrowReaderException(CorruptImageError,"UnrecognizedImageCompression",image)
+          ThrowReaderException(CorruptImageError,UnrecognizedImageCompression,image)
         bytes_per_line=4*(image->columns);
         for (y=(long) image->rows-1; y >= 0; y--)
         {
@@ -1224,12 +1195,12 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         break;
       }
       default:
-        ThrowReaderException(CorruptImageError,"NotABMPImageFile",image)
+        ThrowReaderException(CorruptImageError,ImproperImageHeader,image)
     }
-    LiberateMemory((void **) &pixels);
+    MagickFreeMemory(pixels);
     if (EOFBlob(image))
       {
-        ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+        ThrowException(exception,CorruptImageError,UnexpectedEndOfFile,
           image->filename);
         break;
       }
@@ -1258,7 +1229,8 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         break;
     *magick='\0';
     if (bmp_info.ba_offset != 0)
-      (void) SeekBlob(image,bmp_info.ba_offset,SEEK_SET);
+      if (SeekBlob(image,bmp_info.ba_offset,SEEK_SET) == -1)
+        ThrowReaderException(CorruptImageError,ImproperImageHeader,image)
     (void) ReadBlob(image,2,(char *) magick);
     if (IsBMP(magick,2))
       {
@@ -1331,8 +1303,8 @@ ModuleExport void RegisterBMPImage(void)
   entry->module=AcquireString("BMP");
   entry->adjoin=False;
   entry->seekable_stream=True;
-  (void) RegisterMagickInfo(entry);
 
+  (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("BMP3");
   entry->encoder=(EncoderHandler) WriteBMPImage;
   entry->magick=(MagickHandler) IsBMP;
@@ -1365,6 +1337,8 @@ ModuleExport void RegisterBMPImage(void)
 ModuleExport void UnregisterBMPImage(void)
 {
   (void) UnregisterMagickInfo("BMP");
+  (void) UnregisterMagickInfo("BMP2");
+  (void) UnregisterMagickInfo("BMP3");
 }
 
 /*
@@ -1412,9 +1386,6 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
   register long
     i,
     x;
@@ -1435,6 +1406,12 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
     scene,
     type;
 
+  const unsigned char
+    *color_profile=0;
+
+  size_t
+    color_profile_length=0;
+
   /*
     Open output image file.
   */
@@ -1445,7 +1422,7 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
   logging=LogMagickEvent(CoderEvent,GetMagickModule(),"enter");
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+    ThrowWriterException(FileOpenError,UnableToOpenFile,image);
   type=4;
   if (LocaleCompare(image_info->magick,"BMP2") == 0)
     type=2;
@@ -1453,20 +1430,26 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
     if (LocaleCompare(image_info->magick,"BMP3") == 0)
       type=3;
   scene=0;
+
+  /*
+    Retrieve color profile from Image (if any)
+  */
+  color_profile=GetImageProfile(image,"ICM",&color_profile_length);
+
   do
   {
     /*
       Initialize BMP raster file header.
     */
-    TransformColorspace(image,RGBColorspace);
-    memset(&bmp_info,0,sizeof(BMPInfo));
+    (void) TransformColorspace(image,RGBColorspace);
+    (void) memset(&bmp_info,0,sizeof(BMPInfo));
     bmp_info.file_size=14+12;
     if (type > 2)
       bmp_info.file_size+=28;
     bmp_info.offset_bits=bmp_info.file_size;
     bmp_info.compression=BI_RGB;
     if ((image->storage_class != DirectClass) && (image->colors > 256))
-      SetImageType(image,TrueColorType);
+      (void) SetImageType(image,TrueColorType);
     if (image->storage_class != DirectClass)
       {
         /*
@@ -1481,10 +1464,10 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
           bmp_info.bits_per_pixel=8;
         bmp_info.number_colors=1 << bmp_info.bits_per_pixel;
         if (image->matte)
-          SetImageType(image,TrueColorMatteType);
+          (void) SetImageType(image,TrueColorMatteType);
         else
           if (bmp_info.number_colors < image->colors)
-            SetImageType(image,TrueColorType);
+            (void) SetImageType(image,TrueColorType);
           else
             {
               bmp_info.file_size+=3*(1 << bmp_info.bits_per_pixel);
@@ -1509,7 +1492,7 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
     bytes_per_line=4*((image->columns*bmp_info.bits_per_pixel+31)/32);
     bmp_info.ba_offset=0;
     have_color_info=(int) ((image->rendering_intent != UndefinedIntent) ||
-      (image->color_profile.length != 0) || (image->gamma != 0.0));
+      (color_profile_length != 0) || (image->gamma != 0.0));
     if (type == 2)
       bmp_info.size=12;
     else
@@ -1526,7 +1509,7 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
           bmp_info.size=108;
           extra_size=68;
           if ((image->rendering_intent != UndefinedIntent) ||
-              (image->color_profile.length != 0))
+              (color_profile_length != 0))
             {
               bmp_info.size=124;
               extra_size+=16;
@@ -1555,82 +1538,62 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
     /*
       Convert MIFF to BMP raster pixels.
     */
-    pixels=(unsigned char *) AcquireMemory(bmp_info.image_size);
+    pixels=MagickAllocateMemory(unsigned char *,bmp_info.image_size);
     if (pixels == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
     switch (bmp_info.bits_per_pixel)
     {
       case 1:
       {
-        register unsigned char
-          bit,
-          byte;
+          ExportPixelAreaOptions
+            export_options;
 
-        /*
-          Convert PseudoClass image to a BMP monochrome image.
-        */
-        for (y=0; y < (long) image->rows; y++)
-        {
-          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-          if (p == (const PixelPacket *) NULL)
-            break;
-          indexes=GetIndexes(image);
-          q=pixels+(image->rows-y-1)*bytes_per_line;
-          bit=0;
-          byte=0;
-          for (x=0; x < (long) image->columns; x++)
-          {
-            byte<<=1;
-            byte|=indexes[x] ? 0x01 : 0x00;
-            bit++;
-            if (bit == 8)
-              {
-                *q++=byte;
-                bit=0;
-                byte=0;
-              }
-           }
-         if (bit != 0)
-           *q++=byte << (8-bit);
-         if (image->previous == (Image *) NULL)
-           if (QuantumTick(y,image->rows))
-             if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
-               break;
-        }
-        break;
+          /*
+            Convert PseudoClass image to a BMP monochrome image.
+          */
+          ExportPixelAreaOptionsInit(&export_options);
+          export_options.pad_bytes=(bytes_per_line - ((image->columns+7)/8));
+          export_options.pad_value=0x00;
+          for (y=0; y < (long) image->rows; y++)
+            {
+              p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+              if (p == (const PixelPacket *) NULL)
+                break;
+              q=pixels+(image->rows-y-1)*bytes_per_line;
+              if (ExportImagePixelArea(image,IndexQuantum,1,q,&export_options,0)
+                  == MagickFail)
+                {
+                  break;
+                }
+              if (image->previous == (Image *) NULL)
+                if (QuantumTick(y,image->rows))
+                  if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+                    break;
+            }
+          break;
       }
       case 4:
       {
-        register unsigned char
-          nibble,
-          byte;
+        ExportPixelAreaOptions
+          export_options;
 
         /*
           Convert PseudoClass image to a BMP monochrome image.
         */
+        ExportPixelAreaOptionsInit(&export_options);
+        export_options.pad_bytes=(bytes_per_line - ((image->columns+1)/2));
+        export_options.pad_value=0x00;
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          indexes=GetIndexes(image);
           q=pixels+(image->rows-y-1)*bytes_per_line;
-          nibble=0;
-          byte=0;
-          for (x=0; x < (long) image->columns; x++)
-          {
-            byte<<=4;
-            byte|=(indexes[x] & 0x0f);
-            nibble++;
-            if (nibble == 2)
-              {
-                *q++=byte;
-                nibble=0;
-                byte=0;
-              }
-           }
-         if (nibble != 0)
-           *q++=byte << 4;
+          if (ExportImagePixelArea(image,IndexQuantum,4,q,&export_options,0)
+              == MagickFail)
+            {
+              break;
+            }
          if (image->previous == (Image *) NULL)
            if (QuantumTick(y,image->rows))
              if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
@@ -1640,18 +1603,25 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
       }
       case 8:
       {
+        ExportPixelAreaOptions
+          export_options;
+
         /*
           Convert PseudoClass packet to BMP pixel.
         */
+        ExportPixelAreaOptionsInit(&export_options);
+        export_options.pad_bytes=(bytes_per_line - image->columns);
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          indexes=GetIndexes(image);
           q=pixels+(image->rows-y-1)*bytes_per_line;
-          for (x=0; x < (long) image->columns; x++)
-            *q++=indexes[x];
+          if (ExportImagePixelArea(image,IndexQuantum,8,q,&export_options,0)
+              == MagickFail)
+            {
+              break;
+            }
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
@@ -1680,6 +1650,12 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
               *q++=ScaleQuantumToChar(p->opacity);
             p++;
           }
+          if (bmp_info.bits_per_pixel == 24)
+            {
+              /* initialize padding bytes */
+              for (x=3*image->columns; x < (long) bytes_per_line; x++)
+                *q++=0x00;
+            }
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
@@ -1698,17 +1674,18 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
             Convert run-length encoded raster pixels.
           */
           length=2*(bytes_per_line+2)*(image->rows+2)+2;
-          bmp_data=(unsigned char *) AcquireMemory(length);
-          if (pixels == (unsigned char *) NULL)
+          bmp_data=MagickAllocateMemory(unsigned char *,length);
+          if (bmp_data == (unsigned char *) NULL)
             {
-              LiberateMemory((void **) &pixels);
-              ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
+              MagickFreeMemory(pixels);
+              ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,
                 image)
             }
           bmp_info.file_size-=bmp_info.image_size;
-          bmp_info.image_size=EncodeImage(image,bytes_per_line,pixels,bmp_data);
+          bmp_info.image_size=EncodeImage(image,bytes_per_line,pixels,
+              bmp_data);
           bmp_info.file_size+=bmp_info.image_size;
-          LiberateMemory((void **) &pixels);
+          MagickFreeMemory(pixels);
           pixels=bmp_data;
           bmp_info.compression=BI_RLE8;
         }
@@ -1726,7 +1703,7 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "   Storage class=PseudoClass");
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-          "   Image depth=%lu",image->depth);
+          "   Image depth=%u",image->depth);
         if (image->matte)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "   Matte=True");
@@ -1810,7 +1787,7 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
         (void) WriteBlobLSBLong(image,0x0000ff00L);  /* Green mask */
         (void) WriteBlobLSBLong(image,0x000000ffL);  /* Blue mask */
         (void) WriteBlobLSBLong(image,0xff000000UL);  /* Alpha mask */
-        (void) WriteBlobLSBLong(image,0x00000001L);  /* CSType==Calib. RGB */
+        (void) WriteBlobLSBLong(image,0x00000001L);   /* CSType==Calib. RGB */
         (void) WriteBlobLSBLong(image,
           (long) image->chromaticity.red_primary.x*0x3ffffff);
         (void) WriteBlobLSBLong(image,
@@ -1832,11 +1809,12 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
         (void) WriteBlobLSBLong(image,
           (long) (1.000f-(image->chromaticity.blue_primary.x
           +image->chromaticity.blue_primary.y)*0x3ffffff));
+
         (void) WriteBlobLSBLong(image,(long) bmp_info.gamma_scale.x*0xffff);
         (void) WriteBlobLSBLong(image,(long) bmp_info.gamma_scale.y*0xffff);
         (void) WriteBlobLSBLong(image,(long) bmp_info.gamma_scale.z*0xffff);
         if ((image->rendering_intent != UndefinedIntent) ||
-           (image->color_profile.length != 0))
+           (color_profile_length != 0))
           {
             long
               intent;
@@ -1885,14 +1863,17 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
         */
         if (logging)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-            "  Colormap: %ld entries",image->colors);
-        bmp_colormap=(unsigned char *)
-          AcquireMemory(4*(1 << bmp_info.bits_per_pixel));
+            "  Colormap: %u entries",image->colors);
+        bmp_colormap=MagickAllocateMemory(unsigned char *,
+          (size_t) (4*(1L << bmp_info.bits_per_pixel)));
         if (bmp_colormap == (unsigned char *) NULL)
-          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
+          {
+            MagickFreeMemory(pixels);
+            ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,
+              image);
+          }
         q=bmp_colormap;
-        for (i=0; i < (long) image->colors; i++)
+        for (i=0; i < (long) Min(image->colors,bmp_info.number_colors); i++)
         {
           *q++=ScaleQuantumToChar(image->colormap[i].blue);
           *q++=ScaleQuantumToChar(image->colormap[i].green);
@@ -1908,19 +1889,19 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
           if (type > 2)
             *q++=(Quantum) 0x0;
         }
-        if (type == 2)
-          (void) WriteBlob(image,3*(1 << bmp_info.bits_per_pixel),
+        if (type <= 2)
+          (void) WriteBlob(image,3*(1L << bmp_info.bits_per_pixel),
              (char *) bmp_colormap);
           else
-            (void) WriteBlob(image,4*(1 << bmp_info.bits_per_pixel),
+            (void) WriteBlob(image,4*(1L << bmp_info.bits_per_pixel),
               (char *) bmp_colormap);
-        LiberateMemory((void **) &bmp_colormap);
+        MagickFreeMemory(bmp_colormap);
       }
     if (logging)
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
         "  Pixels:  %lu bytes",bmp_info.image_size);
     (void) WriteBlob(image,bmp_info.image_size,(char *) pixels);
-    LiberateMemory((void **) &pixels);
+    MagickFreeMemory(pixels);
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
