@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2003 GraphicsMagick Group
+  Copyright (C) 2003, 2007 GraphicsMagick Group
   Copyright (C) 2002 ImageMagick Studio
  
   This program is covered by multiple licenses, which are described in
@@ -15,9 +15,18 @@
 extern "C" {
 #endif
 
-#if !defined(vms) && !defined(macintosh) && !defined(WIN32)
-# define POSIX
-#endif
+/*
+  Note that the WIN32 and WIN64 definitions are provided by the build
+  configuration rather than the compiler.  Definitions available from
+  the Windows compiler are _WIN32 and _WIN64.
+*/
+#if defined(WIN32) || defined(WIN64)
+#  define MSWINDOWS
+#endif /* defined(WIN32) || defined(WIN64) */
+
+#if !defined(MSWINDOWS)
+#  define POSIX
+#endif /* !defined(MSWINDOWS) */
 
 /*
   Private functions and types which are not part of the published API
@@ -25,17 +34,10 @@ extern "C" {
 */
 #define MAGICK_IMPLEMENTATION 1
 
-#if !defined(_MAGICK_CONFIG_H)
-# define _MAGICK_CONFIG_H
-# if !defined(vms) && !defined(macintosh)
-#  include "magick/magick_config.h"
-# else
-#  include "magick_config.h"
-# endif
-# if defined(__cplusplus) || defined(c_plusplus)
+#include "magick/magick_config.h"
+#if defined(__cplusplus) || defined(c_plusplus)
 #  undef inline
-# endif
-#endif
+#endif /* defined(__cplusplus) || defined(c_plusplus) */
 
 /*
   Support library symbol prefixing
@@ -50,7 +52,7 @@ extern "C" {
     under AIX unless STDC is defined.
   */
 #  define STDC
-#endif
+#endif /* !defined(const) */
 
 /**
  ** Borland C++ Builder DLL compilation defines
@@ -63,62 +65,55 @@ extern "C" {
 #  define SupportMagickModules
 #endif 
 
-#if defined(WIN32) && !defined(__CYGWIN__)
-# if defined(_MT) && defined(_DLL) && !defined(_MAGICKDLL_) && !defined(_LIB)
-#  define _MAGICKDLL_
-# endif
-# if defined(_MAGICKDLL_)
-#  if defined(_VISUALC_)
-#   pragma warning( disable: 4273 )  /* Disable the dll linkage warnings */
+#if defined(MSWINDOWS) && !defined(__CYGWIN__)
+#  if defined(_MT) && defined(_DLL) && !defined(_MAGICKDLL_) && !defined(_LIB)
+#    define _MAGICKDLL_
 #  endif
-#  if !defined(_MAGICKLIB_)
-#   define MagickExport  __declspec(dllimport)
-#   if defined(_VISUALC_)
-#    pragma message( "Magick lib DLL import interface" )
-#   endif
+#  if defined(_MAGICKDLL_)
+#    if defined(_VISUALC_)
+#      pragma warning( disable: 4273 )  /* Disable the dll linkage warnings */
+#    endif
+#    if !defined(_MAGICKLIB_)
+#      define MagickExport  __declspec(dllimport)
+#      if defined(_VISUALC_)
+#        pragma message( "Magick lib DLL import interface" )
+#      endif
+#    else
+#      define MagickExport  __declspec(dllexport)
+#      if defined(_VISUALC_)
+#         pragma message( "Magick lib DLL export interface" )
+#      endif
+#    endif
 #  else
-#   define MagickExport  __declspec(dllexport)
-#   if defined(_VISUALC_)
-#    pragma message( "Magick lib DLL export interface" )
-#   endif
+#    define MagickExport
+#    if defined(_VISUALC_)
+#      pragma message( "Magick lib static interface" )
+#    endif
 #  endif
-# else
+#  if defined(_DLL) && !defined(_LIB)
+#    define ModuleExport  __declspec(dllexport)
+#    if defined(_VISUALC_)
+#      pragma message( "Magick module DLL export interface" ) 
+#    endif
+#  else
+#    define ModuleExport
+#    if defined(_VISUALC_)
+#      pragma message( "Magick module static interface" ) 
+#    endif
+#  endif
+#  define MagickGlobal __declspec(thread)
+#  if defined(_VISUALC_)
+#    pragma warning(disable : 4018)
+#    pragma warning(disable : 4244)
+#    pragma warning(disable : 4142)
+#    pragma warning(disable : 4800)
+#    pragma warning(disable : 4786)
+#    pragma warning(disable : 4996) /* function deprecation warnings */
+#  endif
+#else
 #  define MagickExport
-#  if defined(_VISUALC_)
-#   pragma message( "Magick lib static interface" )
-#  endif
-# endif
-
-# if defined(_DLL) && !defined(_LIB)
-#  define ModuleExport  __declspec(dllexport)
-#  if defined(_VISUALC_)
-#   pragma message( "Magick module DLL export interface" ) 
-#  endif
-# else
 #  define ModuleExport
-#  if defined(_VISUALC_)
-#   pragma message( "Magick module static interface" ) 
-#  endif
-
-# endif
-# define MagickGlobal __declspec(thread)
-# if defined(_VISUALC_)
-#  pragma warning(disable : 4018)
-#  pragma warning(disable : 4244)
-#  pragma warning(disable : 4142)
-#  pragma warning(disable : 4800)
-#  pragma warning(disable : 4786)
-# endif
-#else
-# define MagickExport
-# define ModuleExport
-# define MagickGlobal
-#endif
-
-#if defined(__cplusplus) || defined(c_plusplus)
-# define storage_class  c_class
-#else
-# define storage_class  class
+#  define MagickGlobal
 #endif
 
 /*
@@ -127,7 +122,7 @@ extern "C" {
  */
 #define MAGICK_IDBASED_MESSAGES 1
 #if defined(MAGICK_IDBASED_MESSAGES)
-#include "magick/locale_c.h"
+#  include "magick/locale_c.h"
 #endif
 
 /*
@@ -139,7 +134,7 @@ extern "C" {
     Currently disabled since feature only seems to work from
     a DLL
   */
-#  if ((defined(WIN32) && defined(_DLL)) && !defined(__MINGW32__))
+#  if ((defined(MSWINDOWS) && defined(_DLL)) && !defined(__MINGW32__))
 #    define MAGICK_WINDOWS_MESSAGE_TABLES 1
 #  endif
 #endif
@@ -149,31 +144,35 @@ extern "C" {
 
 #include <stdarg.h>
 #include <stdio.h>
-#if defined(WIN32) && defined(_DEBUG)
-#define _CRTDBG_MAP_ALLOC
+#if defined(MSWINDOWS) && defined(_DEBUG)
+#  define _CRTDBG_MAP_ALLOC
 #endif
 #include <stdlib.h>
-#if !defined(WIN32)
-# include <unistd.h>
+#if !defined(MSWINDOWS)
+#  include <unistd.h>
 #else
-# include <direct.h>
-# if !defined(HAVE_STRERROR)
-#  define HAVE_STRERROR
-# endif
+#  include <direct.h>
+#  if !defined(HAVE_STRERROR)
+#    define HAVE_STRERROR
+#  endif
 #endif
 
-#if defined(_FILE_OFFSET_BITS)
-# if (_FILE_OFFSET_BITS == 64)
+/*
+  Use fseeko() and ftello() if they are available since they use
+  'off_t' rather than 'long'.  It is wrong to use fseeko() and
+  ftello() only on systems with special LFS support since some systems
+  (e.g. FreeBSD) support a 64-bit off_t by default.
+*/
+#if defined(HAVE_FSEEKO)
 #  define fseek  fseeko
 #  define ftell  ftello
-# endif
 #endif
 
 #if !defined(ExtendedSignedIntegralType)
-# define ExtendedSignedIntegralType magick_int64_t
+#  define ExtendedSignedIntegralType magick_int64_t
 #endif
 #if !defined(ExtendedUnsignedIntegralType)
-# define ExtendedUnsignedIntegralType magick_uint64_t
+#  define ExtendedUnsignedIntegralType magick_uint64_t
 #endif
 
 #define MagickPassFail unsigned int
@@ -195,75 +194,71 @@ extern "C" {
 #include <signal.h>
 #include <assert.h>
 
-#if defined(WIN32) || defined(POSIX)
-# include <sys/types.h>
-# include <sys/stat.h>
-# if defined(HAVE_FTIME)
-# include <sys/timeb.h>
-# endif
-# if defined(POSIX)
+#include <sys/types.h>
+#include <sys/stat.h>
+
+#if defined(HAVE_FTIME)
+#  include <sys/timeb.h>
+#endif
+
+#if defined(POSIX)
 #  if defined(HAVE_SYS_NDIR_H) || defined(HAVE_SYS_DIR_H) || defined(HAVE_NDIR_H)
-#   define dirent direct
-#   define NAMLEN(dirent) (dirent)->d_namlen
-#   if defined(HAVE_SYS_NDIR_H)
-#    include <sys/ndir.h>
-#   endif
-#   if defined(HAVE_SYS_DIR_H)
-#    include <sys/dir.h>
-#   endif
-#   if defined(HAVE_NDIR_H)
-#    include <ndir.h>
-#   endif
+#    define dirent direct
+#    define NAMLEN(dirent) (dirent)->d_namlen
+#    if defined(HAVE_SYS_NDIR_H)
+#      include <sys/ndir.h>
+#    endif
+#    if defined(HAVE_SYS_DIR_H)
+#      include <sys/dir.h>
+#    endif
+#    if defined(HAVE_NDIR_H)
+#      include <ndir.h>
+#    endif
 #  else
-#   include <dirent.h>
-#   define NAMLEN(dirent) strlen((dirent)->d_name)
+#    include <dirent.h>
+#    define NAMLEN(dirent) strlen((dirent)->d_name)
 #  endif
 #  include <sys/wait.h>
 #  include <pwd.h>
-# endif
-# if !defined(S_ISDIR)
-#  define S_ISDIR(mode) (((mode) & S_IFMT) == S_IFDIR)
-# endif
-# if !defined(S_ISREG)
-#  define S_ISREG(mode) (((mode) & S_IFMT) == S_IFREG)
-# endif
-# include "magick/magick_types.h"
-# include "magick/image.h"
-# include "magick/list.h"
-# if !defined(WIN32)
-#  include <sys/time.h>
-#if defined(HAVE_SYS_TIMES_H)
-#  include <sys/times.h>
-#endif
-# endif
-#else
-# include <types.h>
-# include <stat.h>
-# if defined(macintosh)
-#  include <SIOUX.h>
-#  include <console.h>
-#  include <unix.h>
-# endif
-# include "magick/magick_types.h"
-# include "magick/image.h"
-# include "magick/list.h"
 #endif
 
-#if defined(WIN32)
+#if !defined(S_ISDIR)
+#  define S_ISDIR(mode) (((mode) & S_IFMT) == S_IFDIR)
+#endif
+
+#if !defined(S_ISREG)
+#  define S_ISREG(mode) (((mode) & S_IFMT) == S_IFREG)
+#endif
+
+#include "magick/magick_types.h"
+#include "magick/image.h"
+#include "magick/list.h"
+#include "magick/memory.h"
+
+#if !defined(MSWINDOWS)
+#  include <sys/time.h>
+#  if defined(HAVE_SYS_TIMES_H)
+#    include <sys/times.h>
+#  endif
+#endif
+
+
+#if defined(POSIX)
+# include "magick/unix_port.h"
+#endif /* defined(POSIX) */
+
+#if defined(MSWINDOWS)
 # include "magick/nt_base.h"
-#endif
-#if defined(macintosh)
-# include "magick/mac.h"
-#endif
-#if defined(vms)
-# include "magick/vms.h"
-#endif
-#if defined(HAVE_MMAP_FILEIO) && !defined(WIN32)
+#endif /* defined(MSWINDOWS) */
+
+#if defined(HAVE_MMAP_FILEIO) && !defined(MSWINDOWS)
 # include <sys/mman.h>
 #endif
+
 #if defined(HAVE_PTHREAD)
 # include <pthread.h>
 #endif
+
 #if defined(HAVE_POLL)
 # include <sys/poll.h>
 #endif
@@ -285,74 +280,63 @@ extern "C" {
 /*
   Provide prototypes for several functions which are detected to be
   available, but which do not provide a prototype due to interface
-  standards conformace.
+  standards conformance (or a bug).
 */
-#if defined(HAVE_STRLCPY) && !defined(HAVE_STRLCPY_PROTOTYPE)
+
+#if defined(HAVE_PREAD) && defined(HAVE_DECL_PREAD) && !HAVE_DECL_PREAD
+ssize_t pread(int fildes, void *buf, size_t nbyte, off_t offset);
+#endif
+
+#if defined(HAVE_PWRITE) && defined(HAVE_DECL_PWRITE) && !HAVE_DECL_PWRITE
+ssize_t pwrite(int fildes, const void *buf, size_t nbyte, off_t offset);
+#endif
+
+#if defined(HAVE_STRLCPY) && defined(HAVE_DECL_STRLCPY) && !HAVE_DECL_STRLCPY
 extern size_t strlcpy(char *dst, const char *src, size_t dstsize);
 #endif
 
-#if defined(HAVE_VSNPRINTF) && !defined(HAVE_VSNPRINTF_PROTOTYPE)
+#if defined(HAVE_VSNPRINTF) && defined(HAVE_DECL_VSNPRINTF) && !HAVE_DECL_VSNPRINTF
 extern int vsnprintf(char *s, size_t n, const char *format, va_list ap);
 #endif
 
+/*
+  Some 64-bit int support.
+*/
+#if defined(HAVE_STRTOLL) && (SIZEOF_SIGNED_LONG < 8)
+#  define MagickStrToL64(str,endptr,base) (strtoll(str,endptr,base))
+#else
+#  define MagickStrToL64(str,endptr,base) ((magick_int64_t) strtol(str,endptr,base))
+#endif
+#if defined(HAVE_ATOLL) && (SIZEOF_SIGNED_LONG < 8)
+#  define MagickAtoL64(str) (atoll(str))
+#else
+#  define MagickAtoL64(str) ((magick_int64_t) atol(str))
+#endif
+
 /*
   Review these platform specific definitions.
 */
 #if defined(POSIX)
-# define DirectorySeparator  "/"
-# define DirectoryListSeparator  ':'
-# define EditorOptions  " -title \"Edit Image Comment\" -e vi"
-# define Exit  exit
-# define IsBasenameSeparator(c)  ((c) == '/')
-# define PreferencesDefaults  "~/."
-# define ProcessPendingEvents(text)
-# define ReadCommandlLine(argc,argv)
-# define SetNotifyHandlers
-#else
-
-# if defined(vms)
-#  define ApplicationDefaults  "decw$system_defaults:"
-#  define DirectorySeparator  ""
-#  define DirectoryListSeparator  ';'
-#  define EditorOptions  ""
+#  define DirectorySeparator  "/"
+#  define DirectoryListSeparator  ':'
+#  define EditorOptions  " -title \"Edit Image Comment\" -e vi"
 #  define Exit  exit
-#  define IsBasenameSeparator(c)  (((c) == ']') || ((c) == ':') || ((c) == '/'))
-#  define MagickLibPath  "sys$login:"
-#  define MagickCoderModulesPath  "sys$login:"
-#  define MagickFilterModulesPath  "sys$login:"
-#  define MagickSharePath  "sys$login:"
-#  define PreferencesDefaults  "decw$user_defaults:"
+#  define IsBasenameSeparator(c)  ((c) == '/')
+#  define PreferencesDefaults  "~/."
 #  define ProcessPendingEvents(text)
 #  define ReadCommandlLine(argc,argv)
 #  define SetNotifyHandlers
-# endif
+#endif
 
-# if defined(macintosh)
-#  define ApplicationDefaults  "/usr/lib/X11/app-defaults/"
-#  define DirectorySeparator  ":"
-#  define DirectoryListSeparator  ';'
-#  define EditorOptions ""
-#  define IsBasenameSeparator(c)  ((c) == ':')
-#  define MagickLibPath  ""
-#  define MagickCoderModulesPath  ""
-#  define MagickFilterModulesPath  ""
-#  define MagickSharePath  ""
-#  define PreferencesDefaults  "~/."
-#  define ReadCommandlLine(argc,argv)  argc=ccommand(argv); puts(MagickVersion);
-#  define SetNotifyHandlers \
-    SetErrorHandler(MACErrorHandler); \
-    SetWarningHandler(MACWarningHandler)
-# endif
-
-# if defined(WIN32)
+#if defined(MSWINDOWS)
 #  define DirectorySeparator  "\\"
 #  define DirectoryListSeparator  ';'
 #  define EditorOptions ""
 #  define IsBasenameSeparator(c)  (((c) == '/') || ((c) == '\\'))
 #  define ProcessPendingEvents(text)
-#if !defined(PreferencesDefaults)
-#  define PreferencesDefaults  "~\\."
-#endif /* PreferencesDefaults */
+#  if !defined(PreferencesDefaults)
+#     define PreferencesDefaults  "~\\."
+#  endif /* PreferencesDefaults */
 #  define ReadCommandlLine(argc,argv)
 #  define SetNotifyHandlers \
     SetErrorHandler(NTErrorHandler); \
@@ -362,9 +346,8 @@ extern int vsnprintf(char *s, size_t n, const char *format, va_list ap);
 #  if !defined(HAVE_TIFFCONF_H)
 #    define HAVE_TIFFCONF_H
 #  endif
-# endif
+#endif /* defined(MSWINDOWS) */
 
-#endif
 
 /*
   Define declarations.
@@ -372,8 +355,6 @@ extern int vsnprintf(char *s, size_t n, const char *format, va_list ap);
 #define AbsoluteValue(x)  ((x) < 0 ? -(x) : (x))
 #define False  0
 #define DegreesToRadians(x) (MagickPI*(x)/180.0)
-#define IsGray(color)  \
-  (((color).red == (color).green) && ((color).green == (color).blue))
 #define MagickIncarnate(x)  InitializeMagick(x)
 #define MagickEpsilon  1.0e-12
 #define MagickPI  3.14159265358979323846264338327950288419716939937510
@@ -381,7 +362,8 @@ extern int vsnprintf(char *s, size_t n, const char *format, va_list ap);
 #define Max(x,y)  (((x) > (y)) ? (x) : (y))
 #define Min(x,y)  (((x) < (y)) ? (x) : (y))
 #define QuantumTick(i,span) \
-  ((((i) & 0xff) == 0) || (i == ((magick_int64_t) (span)-1)))
+  ((((i) % ((Max(101,span)-1)/100)) == 0) || \
+    ((magick_int64_t) (i) == ((magick_int64_t) (span)-1)))
 #define RadiansToDegrees(x) (180.0*(x)/MagickPI)
 #define ScaleColor5to8(x)  (((x) << 3) | ((x) >> 2))
 #define ScaleColor6to8(x)  (((x) << 2) | ((x) >> 4))
@@ -415,7 +397,7 @@ extern int vsnprintf(char *s, size_t n, const char *format, va_list ap);
 #  define MAP_FAILED      ((void *) -1)
 #endif
 
-#if defined(HasLTDL) || ( defined(WIN32) && defined(_DLL) )
+#if defined(HasLTDL) || ( defined(MSWINDOWS) && defined(_DLL) )
 #  define SupportMagickModules
 #endif
 
@@ -428,7 +410,7 @@ extern int vsnprintf(char *s, size_t n, const char *format, va_list ap);
 /*
   I/O defines.
 */
-#if defined(WIN32) && !defined(Windows95) && !defined(__BORLANDC__)
+#if defined(MSWINDOWS) && !defined(Windows95) && !defined(__BORLANDC__)
   /* Windows '95 and Borland C do not support _lseeki64 */
 #  define MagickSeek(file,offset,whence)  _lseeki64(file,offset,whence)
 #  define MagickTell(file) _telli64(file)
@@ -438,17 +420,45 @@ extern int vsnprintf(char *s, size_t n, const char *format, va_list ap);
 #endif
 
 #if !defined(HAVE_POPEN) && defined(HAVE__POPEN)
-# define HAVE_POPEN 1
-# define popen _popen
-#endif
+#  define HAVE_POPEN 1
+#  define popen _popen
+#endif /* !defined(HAVE_POPEN) && defined(HAVE__POPEN) */
 
 #if !defined(HAVE_PCLOSE) && defined(HAVE__PCLOSE)
-# define HAVE_PCLOSE 1
-# define pclose _pclose
-#endif
+#  define HAVE_PCLOSE 1
+#  define pclose _pclose
+#endif /* !defined(HAVE_PCLOSE) && defined(HAVE__PCLOSE) */
+
+#if defined(HAVE__EXIT)
+#  define SignalHandlerExit _exit
+#else
+#  define SignalHandlerExit Exit
+#endif /* defined(HAVE__EXIT) */
+
+
+/*
+  Image const declarations.
+*/
+extern MagickExport const char
+  *BackgroundColor,
+  *BorderColor,
+  *DefaultTileFrame,
+  *DefaultTileGeometry,
+  *DefaultTileLabel,
+  *ForegroundColor,
+  *LoadImageText,
+  *LoadImagesText,
+  *MatteColor,
+  *PSDensityGeometry,
+  *PSPageGeometry,
+  *SaveImageText,
+  *SaveImagesText;
+
+extern MagickExport const unsigned long
+  DefaultCompressionQuality;
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
-#endif
+#endif /* defined(__cplusplus) || defined(c_plusplus) */
 
-#endif
+#endif /* ifndef _MAGICK_STUDIO_H */
